<?php
/**
 * Plugin Name: Collab Notes
 * Description: A plugin to add admin-only notes to Pages and Posts.
 * Version: 1.3
 * Author: Paolo Sartorio
 * License: GPLv2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Requires at least: 5.6
 * Requires PHP: 7.2
 */

// Add a metabox for the notes in Pages and Posts
function collab_notes_add_metabox() {
    if (!collab_notes_user_can_edit()) {
        return;
    }
    add_meta_box(
        'collab_notes_metabox',
        'Collab Note',
        'collab_notes_render_metabox',
        ['post', 'page'],
        'side',
        'high'
    );
}
add_action('add_meta_boxes', 'collab_notes_add_metabox');

// Render the metabox
function collab_notes_render_metabox($post) {
    $note = get_post_meta($post->ID, '_collab_note', true);
    wp_nonce_field('collab_notes_save_note', 'collab_notes_nonce');
    echo '<textarea name="collab_note" style="width: 100%; height: 100px; padding: 8px; border-radius: 4px; border: 1px solid #ddd;">' . esc_textarea($note) . '</textarea>';
}

// Save the note when the post is saved
function collab_notes_save_post($post_id) {
    if (!isset($_POST['collab_notes_nonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['collab_notes_nonce'])), 'collab_notes_save_note')) {
        return;
    }
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
        return;
    }
    if (!collab_notes_user_can_edit()) {
        return;
    }
    if (isset($_POST['collab_note'])) {
        $note = wp_kses_post(wp_unslash($_POST['collab_note']));
        update_post_meta($post_id, '_collab_note', $note);

        // Save note history
        $note_history = get_post_meta($post_id, '_collab_note_history', true);
        if (!is_array($note_history)) {
            $note_history = [];
        }
        $note_history[] = [
            'note' => $note,
            'timestamp' => current_time('mysql'),
            'user' => get_current_user_id(),
        ];
        update_post_meta($post_id, '_collab_note_history', $note_history);
    }
}
add_action('save_post', 'collab_notes_save_post');

// Add a column for notes in the admin list table
function collab_notes_add_column($columns) {
    $columns['collab_note'] = 'Collab Note';
    return $columns;
}
add_filter('manage_posts_columns', 'collab_notes_add_column');
add_filter('manage_pages_columns', 'collab_notes_add_column');

// Populate the notes column
function collab_notes_populate_column($column, $post_id) {
    if ('collab_note' === $column) {
        $note = get_post_meta($post_id, '_collab_note', true);
        if (!empty($note) && collab_notes_user_can_view()) {
            echo '<span class="dashicons dashicons-media-text" title="' . esc_attr($note) . '" aria-label="' . esc_attr($note) . '"></span>';
        }
    }
}
add_action('manage_posts_custom_column', 'collab_notes_populate_column', 10, 2);
add_action('manage_pages_custom_column', 'collab_notes_populate_column', 10, 2);

// Register a new menu for Collab Notes
function collab_notes_register_menu() {
    add_menu_page(
        'Collab Notes',
        'Collab Notes',
        'manage_options',
        'collab-notes',
        'collab_notes_render_list_page',
        'dashicons-media-text',
        6
    );

    // Add submenu for settings
    add_submenu_page(
        'collab-notes',
        'Collab Notes Settings',
        'Settings',
        'manage_options',
        'collab-notes-settings',
        'collab_notes_render_settings_page'
    );
}
add_action('admin_menu', 'collab_notes_register_menu');

// Render the list page for all notes
function collab_notes_render_list_page() {
    if (!current_user_can('manage_options')) {
        return;
    }

    // Query posts with notes
    $args = [
        'post_type' => ['post', 'page'],
        'meta_key' => '_collab_note',
        'meta_value' => '',
        'meta_compare' => '!=',
        'posts_per_page' => -1,
    ];
    $query = new WP_Query($args);

    echo '<div class="wrap">
        <h1>Collab Notes</h1>
        <table class="wp-list-table widefat fixed striped">
            <thead>
                <tr>
                    <th>Post/Page Title</th>
                    <th>Note</th>
                    <th>Type</th>
                </tr>
            </thead>
            <tbody>';

    if ($query->have_posts()) {
        while ($query->have_posts()) {
            $query->the_post();
            $note = get_post_meta(get_the_ID(), '_collab_note', true);
            echo '<tr>
                <td><a href="' . esc_url(get_edit_post_link(get_the_ID())) . '">' . esc_html(get_the_title()) . '</a></td>
                <td>' . esc_html($note) . '</td>
                <td>' . esc_html(get_post_type_object(get_post_type())->labels->singular_name) . '</td>
            </tr>';
        }
        wp_reset_postdata();
    } else {
        echo '<tr><td colspan="3">No notes found.</td></tr>';
    }

    echo '</tbody>
        </table>
    </div>';
}

// Render settings page
function collab_notes_render_settings_page() {
    if (!current_user_can('manage_options')) {
        return;
    }

    if (isset($_POST['collab_notes_settings_nonce']) && wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['collab_notes_settings_nonce'])), 'collab_notes_save_settings')) {
        $sanitized_edit_roles = array_map('sanitize_text_field', wp_unslash($_POST['edit_roles'] ?? []));
        $sanitized_view_roles = array_map('sanitize_text_field', wp_unslash($_POST['view_roles'] ?? []));

        update_option('collab_notes_edit_roles', $sanitized_edit_roles);
        update_option('collab_notes_view_roles', $sanitized_view_roles);

        echo '<div class="updated"><p>Settings saved.</p></div>';
    }

    $roles = wp_roles()->roles;
    $edit_roles = get_option('collab_notes_edit_roles', ['administrator']);
    $view_roles = get_option('collab_notes_view_roles', ['administrator']);

    echo '<div class="wrap">
        <h1>Collab Notes Settings</h1>
        <form method="post">';
    wp_nonce_field('collab_notes_save_settings', 'collab_notes_settings_nonce');
    
    echo '<h2>Roles allowed to add/edit notes:</h2>
        <ul>';
    foreach ($roles as $role => $details) {
        echo '<li>
            <label>
                <input type="checkbox" name="edit_roles[]" value="' . esc_attr($role) . '" ' . checked(in_array($role, $edit_roles), true, false) . '>
                ' . esc_html($details['name']) . '
            </label>
        </li>';
    }
    echo '</ul>';

    echo '<h2>Roles allowed to view notes:</h2>
        <ul>';
    foreach ($roles as $role => $details) {
        echo '<li>
            <label>
                <input type="checkbox" name="view_roles[]" value="' . esc_attr($role) . '" ' . checked(in_array($role, $view_roles), true, false) . '>
                ' . esc_html($details['name']) . '
            </label>
        </li>';
    }
    echo '</ul>';

    echo '<p><input type="submit" class="button-primary" value="Save Changes"></p>
        </form>';

    // 🔥 Buy Me a Coffee button
    $image_url = plugin_dir_url(__FILE__) . 'assets/buymeacoffee.png';
    echo '<div style="margin-top: 20px; text-align: center;">
        <p>If you like this plugin, consider supporting development:</p>
        <a href="https://www.buymeacoffee.com/paolosartorio" target="_blank">
            <img src="' . esc_url($image_url) . '" alt="Buy Me A Coffee" style="height: 60px !important;width: 217px !important;">
        </a>
    </div>';

    echo '</div>';
}

// Check if the user can edit notes
function collab_notes_user_can_edit() {
    $edit_roles = get_option('collab_notes_edit_roles', ['administrator']);
    foreach ($edit_roles as $role) {
        if (current_user_can($role)) {
            return true;
        }
    }
    return false;
}

// Check if the user can view notes
function collab_notes_user_can_view() {
    $view_roles = get_option('collab_notes_view_roles', ['administrator']);
    foreach ($view_roles as $role) {
        if (current_user_can($role)) {
            return true;
        }
    }
    return false;
}